defmodule DecentralisedBookIndex.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  @impl true
  def start(_type, _args) do
    children = [
      DecentralisedBookIndexWeb.Telemetry,
      DecentralisedBookIndex.Repo,
      {DNSCluster,
       query: Application.get_env(:decentralised_book_index, :dns_cluster_query) || :ignore},
      {Phoenix.PubSub, name: DecentralisedBookIndex.PubSub},
      # Start the Finch HTTP client for sending emails
      {Finch, name: DecentralisedBookIndex.Finch},
      # Start a worker by calling: DecentralisedBookIndex.Worker.start_link(arg)
      # {DecentralisedBookIndex.Worker, arg},
      # Start to serve requests, typically the last entry
      DecentralisedBookIndexWeb.Endpoint,
      {AshAuthentication.Supervisor, [otp_app: :decentralised_book_index]}
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: DecentralisedBookIndex.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  @impl true
  def config_change(changed, _new, removed) do
    DecentralisedBookIndexWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
